/**********************************************************************
  Filename    : Camera Remote Bluetooth IoT
  Description : The esp32 controls remote android camera via bluetooth.
                This version is for the M5Stack Atom Lite (esp32 pico) controller.
  Function    : Photo, video recording, flash mode, and switch camera (v2)
  Auther      : www.busywww.com   busyweb@gmail.com
  Modification: 2022/02/09
**********************************************************************/
#include "soc/soc.h"
#include "soc/rtc_cntl_reg.h"

#include <M5Atom.h>
#include "BluetoothSerial.h"
#include "string.h"

BluetoothSerial SerialBT;

int APP_MODE = 5;             //0: Switched Mode, 1: Joystick Mode, 5: M5
String dataValues[15];    

int ACTION_STATUS_NONE = 0;
int ACTION_STATUS_BTN_ACTION = 1;
int ACTION_STATUS_BTN_PRESSED = 2;
int ACTION_STATUS_BTN_LONG_PRESSED = 3;
int ACTION_STATUS_BTN_RELEASED = 4;

int ACTION_STATUS = ACTION_STATUS_NONE;

int FLASH_STATUS = 0;         //1: torch flash, 0: auto
int RECORDING_STATUS = 0;

int MODE_NONE = 0;
int MODE_PHOTO = 1;           //1: photo mode, 0: video mode
int MODE_VIDEO = 2;           //1: video mode, 0: photo mode
int MODE_VIDEO_RECORDING = 22; // recording video led flashing
int MODE_FLASH = 3;           //  1: torch flash, 0: auto
int MODE_CURRENT = MODE_PHOTO;


// Bluetooth Status
int CONNECTED = 0;

uint8_t DisBuff[2 + 5 * 5 * 3]; //Used to store RBG color values.
void setBuff(uint8_t Rdata, uint8_t Gdata, uint8_t Bdata){  //Set the colors of LED, and save the relevant data to DisBuff[].
    DisBuff[0] = 0x05;
    DisBuff[1] = 0x05;
    for (int i = 0; i < 25; i++){
        DisBuff[2 + i * 3 + 0] = Rdata;
        DisBuff[2 + i * 3 + 1] = Gdata;
        DisBuff[2 + i * 3 + 2] = Bdata;
    }
}
void displayLed() {
  if (MODE_CURRENT == MODE_NONE) {
     setBuff(0x20, 0x20, 0x20);
  } else if (MODE_CURRENT == MODE_PHOTO) {
     setBuff(0x00, 0x40, 0x00);
  } else if (MODE_CURRENT == MODE_VIDEO) {
     setBuff(0x40, 0x00, 0x00);
  } else if (MODE_CURRENT == MODE_FLASH) {
     setBuff(0x00, 0x00, 0x40);
  }  
  M5.dis.displaybuff(DisBuff);    //Display the DisBuff color on the LED.    
}

void setup() {

  MODE_CURRENT = MODE_NONE;
  
  M5.begin(true, false, true); // init M5Atom
  delay(10);
  displayLed();
  
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0); //disable brownout detector   ds

  SerialBT.begin("Camera Remote Bluetooth"); //Bluetooth device name
  SerialBT.register_callback(callback);
  
  Serial.println("\nThe device started, now you can pair it with bluetooth!");
  
  delay(500);
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 1); //enable brownout detector

}

void callback(esp_spp_cb_event_t event, esp_spp_cb_param_t *param) {
  if(event == ESP_SPP_SRV_OPEN_EVT){
    Serial.println("Client Connected");
    //digitalWrite(LED,HIGH);
    CONNECTED = true;

    MODE_CURRENT = MODE_PHOTO;
    displayLed();
  }
 
  if(event == ESP_SPP_CLOSE_EVT ){
    Serial.println("Client disconnected");
    //digitalWrite(LED,LOW);
    CONNECTED = false;
    
    MODE_CURRENT = MODE_NONE;
    displayLed();
  }
}

void processAction() {
  //You need to add m5.update () to read the state of the key, see System for details
  M5.update();
  
//int ACTION_STATUS_NONE = 0;
//int ACTION_STATUS_BTN_ACTION = 1;
//int ACTION_STATUS_BTN_PRESSED = 2;
//int ACTION_STATUS_BTN_LONE_PRESSED = 3;
//int ACTION_STATUS_BTN_RELEASED = 4;

  if (ACTION_STATUS == ACTION_STATUS_NONE || ACTION_STATUS == ACTION_STATUS_BTN_PRESSED) {

    if (M5.Btn.pressedFor(2000)) {
      // If the button is pressed for more than 2 seconds
      if (MODE_CURRENT == MODE_PHOTO) {
        MODE_CURRENT = MODE_VIDEO;
      } else if (MODE_CURRENT == MODE_VIDEO) {
        MODE_CURRENT = MODE_FLASH;
      } else if (MODE_CURRENT == MODE_FLASH) {
        MODE_CURRENT = MODE_PHOTO;
      }
      Serial.println("Button long pressed: " + String(MODE_CURRENT));

      displayLed();
      
      ACTION_STATUS = ACTION_STATUS_BTN_LONG_PRESSED;
    } else {

      if (ACTION_STATUS == ACTION_STATUS_NONE) {
        if (M5.Btn.isPressed()) {
          Serial.println("Button pressed...");
          ACTION_STATUS = ACTION_STATUS_BTN_PRESSED;
        }
      } else if (ACTION_STATUS == ACTION_STATUS_BTN_PRESSED) {
        if (M5.Btn.isReleased()) {
          Serial.println("Button released from Pressed.");
          // do remote action
          doAction();

          delay(100);
          ACTION_STATUS = ACTION_STATUS_NONE;
        }       
      }
      
    }
  
  } else if (ACTION_STATUS == ACTION_STATUS_BTN_LONG_PRESSED) {

    if (M5.Btn.isReleased()) {
      Serial.println("Button released from Long Pressed.");
      
      displayLed();

      doModeAction();

      ACTION_STATUS = ACTION_STATUS_NONE;
    }
    
  }  
}

void doAction() {
  
//int MODE_NONE = 0;
//int MODE_PHOTO = 1;           //1: photo mode, 0: video mode
//int MODE_VIDEO = 2;           //1: video mode, 0: photo mode
//int MODE_VIDEO_RECORDING = 22; // recording video led flashing
//int MODE_FLASH = 3;   

  String actionString = "iot:";
  String displayType = "none";
  
  if (MODE_CURRENT == MODE_PHOTO) {
    actionString += "0:0:0:1:0:0:" + displayType;
  } else if (MODE_CURRENT == MODE_VIDEO) {
    actionString += "0:0:0:1:0:0:" + displayType;
  } else if (MODE_CURRENT == MODE_VIDEO_RECORDING) {
    actionString += "0:0:0:1:0:0:" + displayType;
  } else if (MODE_CURRENT == MODE_FLASH) {
    FLASH_STATUS = (FLASH_STATUS == 1 ? 0 : 1);
    actionString += "0:0:" + String(FLASH_STATUS) + ":0:0:0:" + displayType;
  }
  Serial.println("doAction: " + actionString);

  if (CONNECTED) {
    SerialBT.println(actionString);
    SerialBT.write(13);
    SerialBT.write(10);   

    delay(250);
  }
  
}

void doModeAction() {
  
//int MODE_NONE = 0;
//int MODE_PHOTO = 1;           //1: photo mode, 0: video mode
//int MODE_VIDEO = 2;           //1: video mode, 0: photo mode
//int MODE_VIDEO_RECORDING = 22; // recording video led flashing
//int MODE_FLASH = 3;   

  String actionString = "iot:";
  String displayType = "none";
  
  if (MODE_CURRENT == MODE_PHOTO) {
    actionString += "1:0:0:0:0:0:" + displayType;
  } else if (MODE_CURRENT == MODE_VIDEO) {
    actionString += "0:1:0:0:0:0:" + displayType;
  } else if (MODE_CURRENT == MODE_FLASH) {
    //actionString += "0:0:1:0:0:0:" + displayType;
  } 
  Serial.println(actionString);

  if (CONNECTED) {
    SerialBT.println(actionString);
    SerialBT.write(13);
    SerialBT.write(10);   

    delay(250);
  }
  
}

void clearDataValues() {
  int numberOfValues = 10;    //(int)sizeof(dataValues)/sizeof("hello");
  for (int i = 0; i < numberOfValues; i++) {
    dataValues[i] = "";
  }
}

// parse the remote data headers and values
// split data string into array
// delimeter: ":"
void parseData(String dataString) {
  clearDataValues();

  // data delimeter: ":"
  char delimeter = ':';
  int findStart = 0;
  int findIndex = 0;
  int findCount = 0;

  findIndex = dataString.indexOf(delimeter, findStart);
  if (findIndex > 0) {
    dataValues[findCount] = dataString.substring(findStart, findIndex);
    findCount ++;
    findStart = findIndex + 1;

    while (findIndex > 0) {
      findIndex = dataString.indexOf(delimeter, findStart);
      if (findIndex > 0) {
        dataValues[findCount] = dataString.substring(findStart, findIndex);
        findCount ++;
        findStart = findIndex + 1;        
      }
    } 
  }

  String dataName = dataValues[0];

  if (dataName.equalsIgnoreCase("iot")) {
    // current remote camera status
    // iot:0:1:0:0:ok:0:0:192:
    
    String statusPhoto = dataValues[1];
    int photoValue = statusPhoto.toInt();
    String statusVideo = dataValues[2];
    int videoValue = statusVideo.toInt();
    String statusFlash = dataValues[3];
    int flashValue = statusFlash.toInt();
    String statusAction = dataValues[4];
    int actionValue = statusAction.toInt();
    String statusResult = dataValues[5];
    int resultValue = statusResult.toInt();
    String statusDisplay = dataValues[6];  // none, text, image
    String previewSize = dataValues[7];
    String pixelsOnData = dataValues[8];    

    FLASH_STATUS = flashValue;

    if (MODE_CURRENT != MODE_FLASH) {
      if (photoValue == 1) {
        if (MODE_CURRENT != MODE_PHOTO) {
          MODE_CURRENT = MODE_PHOTO;
          displayLed();
        }
      } else if (videoValue == 1) {
        if (MODE_CURRENT != MODE_VIDEO) {
          MODE_CURRENT = MODE_VIDEO;
          displayLed();
        }
      }      
    }

  } else if (dataName.equalsIgnoreCase("video")) {
    // video recording status
    // video:2:, video:1:, ...
    if (MODE_CURRENT != MODE_VIDEO) {
      MODE_CURRENT = MODE_VIDEO;
      displayLed();
    } else {
      // show video recording status
      // blink led
      if (RECORDING_STATUS == 0) {
        setBuff(0x10, 0x00, 0x00);
        M5.dis.displaybuff(DisBuff);    //Display the DisBuff color on the LED.  
        RECORDING_STATUS = 1;
      } else {
        setBuff(0x40, 0x00, 0x00);
        M5.dis.displaybuff(DisBuff);    //Display the DisBuff color on the LED. 
        RECORDING_STATUS = 0; 
      }     
    }
    
  }
  
}

void loop() {

  if (CONNECTED) {
    
    while(SerialBT.available())
    {
      String received = SerialBT.readStringUntil('\n');

      Serial.println(received);

      parseData(received);
    }

    processAction();

  } else { //end of: if (CONNECTED)

    // not connected
    
  }

  delay(10);
}
